#include <stdio.h>
#include "system.h"
#include "string.h"
#include "altera_avalon_pio_regs.h"
#include <unistd.h>
#include "io.h"
#include "altera_avalon_jtag_uart_regs.h"

#define NUMBER_OF_MS_PER_HOUR 3600000
#define NUMBER_OF_MS_PER_MINUTE 60000
#define NUMBER_OF_MS_PER_SECOND 1000
#define NUMBER_OF_LINKS 1
#define NUMBER_OF_LANES 10 
#define NUMBER_OF_QUADS 1
#define MULTIPLIER 65536 // 2^16

#define DEBUG 0
#define TOP 0
#define BOTTOM 1
#define RESET_CYCLES 5000
#define STOP 1
#define HISTOGRAM_SIZE 8

#define VOD 0
#define PRE_TAP 1
#define POST_TAP1 2
#define POST_TAP2 3
#define EQ_GAIN 16 
#define EQ_CTRL 17 
#define PRECDR 32 
#define POSTCDR 33 

#define VOD_200MV 10
#define VOD_400MV 20
#define VOD_600MV 30
#define VOD_700MV 35
#define VOD_800MV 40
#define VOD_900MV 45
#define VOD_1000MV 50
#define VOD_1200MV 60

#define PRE_TAP_15N 1
#define PRE_TAP_14N 2
#define PRE_TAP_13N 3
#define PRE_TAP_12N 4
#define PRE_TAP_11N 5
#define PRE_TAP_10N 6
#define PRE_TAP_9N 7
#define PRE_TAP_8N 8
#define PRE_TAP_7N 9
#define PRE_TAP_6N 10
#define PRE_TAP_5N 11
#define PRE_TAP_4N 12
#define PRE_TAP_3N 13
#define PRE_TAP_2N 14
#define PRE_TAP_1N 15
#define PRE_TAP_0  0 //0 or 16 for this setting
#define PRE_TAP_1  17
#define PRE_TAP_2  18
#define PRE_TAP_3  19
#define PRE_TAP_4  20
#define PRE_TAP_5  21
#define PRE_TAP_6  22
#define PRE_TAP_7  23
#define PRE_TAP_8  24
#define PRE_TAP_9  25
#define PRE_TAP_10  26
#define PRE_TAP_11  27
#define PRE_TAP_12  28
#define PRE_TAP_13  29
#define PRE_TAP_14  30
#define PRE_TAP_15  31

#define POST_TAP1_0 0
#define POST_TAP1_1 1
#define POST_TAP1_2 2
#define POST_TAP1_3 3
#define POST_TAP1_4 4
#define POST_TAP1_5 5
#define POST_TAP1_6 6
#define POST_TAP1_7 7
#define POST_TAP1_8 8
#define POST_TAP1_9 9
#define POST_TAP1_10 10
#define POST_TAP1_11 11
#define POST_TAP1_12 12
#define POST_TAP1_13 13
#define POST_TAP1_14 14
#define POST_TAP1_15 15
#define POST_TAP1_16 16
#define POST_TAP1_17 17
#define POST_TAP1_18 18
#define POST_TAP1_19 19
#define POST_TAP1_20 20
#define POST_TAP1_21 21
#define POST_TAP1_22 22
#define POST_TAP1_23 23
#define POST_TAP1_24 24
#define POST_TAP1_25 25
#define POST_TAP1_26 26
#define POST_TAP1_27 27
#define POST_TAP1_28 28
#define POST_TAP1_29 29
#define POST_TAP1_30 30
#define POST_TAP1_31 31


#define POST_TAP2_15N 1
#define POST_TAP2_14N 2
#define POST_TAP2_13N 3
#define POST_TAP2_12N 4
#define POST_TAP2_11N 5
#define POST_TAP2_10N 6
#define POST_TAP2_9N 7
#define POST_TAP2_8N 8
#define POST_TAP2_7N 9
#define POST_TAP2_6N 10
#define POST_TAP2_5N 11
#define POST_TAP2_4N 12
#define POST_TAP2_3N 13
#define POST_TAP2_2N 14
#define POST_TAP2_1N 15
#define POST_TAP2_0  0 //0 or 16 for this setting
#define POST_TAP2_1  17
#define POST_TAP2_2  18
#define POST_TAP2_3  19
#define POST_TAP2_4  20
#define POST_TAP2_5  21
#define POST_TAP2_6  22
#define POST_TAP2_7  23
#define POST_TAP2_8  24
#define POST_TAP2_9  25
#define POST_TAP2_10  26
#define POST_TAP2_11  27
#define POST_TAP2_12  28
#define POST_TAP2_13  29
#define POST_TAP2_14  30
#define POST_TAP2_15  31

#define EQ_CTRL_0 0
#define EQ_CTRL_1 1
#define EQ_CTRL_2 2
#define EQ_CTRL_3 3
#define EQ_CTRL_4 4
#define EQ_CTRL_5 5
#define EQ_CTRL_6 6
#define EQ_CTRL_7 7
#define EQ_CTRL_8 8
#define EQ_CTRL_9 9
#define EQ_CTRL_10 10
#define EQ_CTRL_11 11
#define EQ_CTRL_12 12
#define EQ_CTRL_13 13
#define EQ_CTRL_14 14
#define EQ_CTRL_15 15

#define EQ_GAIN_0 0
#define EQ_GAIN_1 1
#define EQ_GAIN_2 2
#define EQ_GAIN_3 3
#define EQ_GAIN_4 4
#define EQ_GAIN_5 5
#define EQ_GAIN_6 6
#define EQ_GAIN_7 7



/* Functions */

/*-----------------------------------------------
 * input_char()
 * 
 * Gets a single input char for menu prompt
 * 
 *   ******** Added by Guy for Menu inputs
 *   ******** Changed by Maurizio for Nios II SDK Shell support
 * 
 * ----------------------------------------------*/
char input_char(void)
 {
 char in_char, rx_char;
 
   rx_char = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
         rx_char = in_char;
     else if (in_char == '\n')
         break;
   };
 return (rx_char);
 }

int input_number(void)
 {
 char in_char,LSB;
 int temp, number;
 
   LSB = 0;
   temp = 0;
   number = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;        
         LSB = in_char;
        }
     else if (in_char == '\n')
         break;
          
            switch (LSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
         
    
          number  = temp;         
   };
 return (number);
 } 
 
int input_byte(void)
 {
 char in_char, MSB,LSB;
 int temp, High_nibble, Low_nibble, Byte;
 
   MSB = 0;
   LSB = 0;
   temp = 0;
   Byte = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;
         MSB = LSB;         
         LSB = in_char;
        }
     else if (in_char == '\n')
         break;
     switch (MSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          High_nibble = temp;
          
            switch (LSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          Low_nibble = temp;  
    
          Byte = (High_nibble << 4) | Low_nibble;         
   };
 return (Byte);
 } 


 
int input_word(void)
 {
 char in_char, N3,N2,N1,N0;
 int temp, N3_int, N2_int,N1_int,N0_int, Word;
 
   N3 = 0;
   N2 = 0;
   N1 = 0;
   N0 = 0;         
   temp = 0;
   Word = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;
         N3 = N2;
         N2 = N1;
         N1 = N0;
         N0 = in_char;
        }
     else if (in_char == '\n')
         break;
     switch (N3)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N3_int = temp;
          
         switch (N2)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N2_int = temp;  

         switch (N1)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N1_int = temp;  

         switch (N0)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N0_int = temp;                
          Word = (N3_int << 12) | (N2_int << 8) | (N1_int << 4) | N0_int;         
   };
 return (Word);
 } 
 
 void write_reconfiguration(int logical_channel_address,int tx_rx_word_offset, int reconfig_data)
 {
	 int Control_and_Status;
//	 int Error;
	 int Busy;
//	 int readback;	 

	 do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_BASE,0x00A);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);
	  
	  
	  IOWR(RECONFIG_MGMT_BASE,0x008,logical_channel_address);
	  
	  IOWR(RECONFIG_MGMT_BASE,0x00B,tx_rx_word_offset);  
	  
	  IOWR(RECONFIG_MGMT_BASE,0x00C,reconfig_data);	
	  
	  IOWR(RECONFIG_MGMT_BASE,0x00A,0x0001); // Perform a write operation

    do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_BASE,0x00A);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);  
	  
	  Control_and_Status = IORD(RECONFIG_MGMT_BASE,0x00A);
//	  Error = 0x0001 & (Control_and_Status >> 9); 	  
}  

void write_reconfiguration_EyeQ(int logical_channel_address,int tx_rx_word_offset, int reconfig_data)
 {
	 int Control_and_Status;
	 //int Error;
	 int Busy;
	 //int readback;	 

	 
	  
	  IOWR(RECONFIG_MGMT_BASE,0x010,logical_channel_address);

	  
	  IOWR(RECONFIG_MGMT_BASE,0x013,tx_rx_word_offset);	  
	  
	  IOWR(RECONFIG_MGMT_BASE,0x014,reconfig_data);	
	  
	  IOWR(RECONFIG_MGMT_BASE,0x012,0x0001); // Perform a write operation

    do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_BASE,0x012);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);
		  
	  
	  Control_and_Status = IORD(RECONFIG_MGMT_BASE,0x012);
//	  Error = 0x0001 & (Control_and_Status >> 9); 	  
}  

int read_reconfiguration_EyeQ(int logical_channel_address,int tx_rx_word_offset)
 {
	 int Control_and_Status;
	 //int Error;
	 int Busy;
	 int reconfig_data;
	 //int readback;	 

	  
	  IOWR(RECONFIG_MGMT_BASE,0x010,logical_channel_address);

	  
	  IOWR(RECONFIG_MGMT_BASE,0x013,tx_rx_word_offset);	  
	
	  
	  IOWR(RECONFIG_MGMT_BASE,0x012,0x0002); // Perform a read operation

    do
    {
     Control_and_Status = IORD(RECONFIG_MGMT_BASE,0x012);
	  Busy = 0x0001 & (Control_and_Status >> 8); 
	  } while (Busy == 1);
		  
	  Control_and_Status = IORD(RECONFIG_MGMT_BASE,0x012);
	  //Error = 0x0001 & (Control_and_Status >> 9); 	  
	  
	  reconfig_data = IORD(RECONFIG_MGMT_BASE,0x014); 
	 return(reconfig_data);
} 
  
  
/* Functions */

int main()
{ 
	char rx_char;
  int Control_Reg[NUMBER_OF_QUADS];

  int RefClock_Reg[NUMBER_OF_QUADS];
  int DataClock_Reg[NUMBER_OF_QUADS];
  //int Delay_Reg[2];
  //int Delay_lane[NUMBER_OF_LANES];
  int Bitrate[NUMBER_OF_QUADS];
  int Linerate[NUMBER_OF_QUADS];
  int Latency_measure[NUMBER_OF_QUADS];
  int Latency_memory_measure[HISTOGRAM_SIZE];	
  float Bitrate_temp[NUMBER_OF_QUADS];
  float Linerate_temp[NUMBER_OF_QUADS];
  float Latency_measure_temp[NUMBER_OF_QUADS];
  float Latency_memory_measure_temp[HISTOGRAM_SIZE];	
  int Userdatarate[NUMBER_OF_QUADS];
  float Userdatarate_temp[NUMBER_OF_QUADS];
  int Refclockmultiplier[NUMBER_OF_QUADS];
  int Channel_Reg[NUMBER_OF_LINKS];
  int Counter_1ms_Reg[NUMBER_OF_QUADS];
  float BER[NUMBER_OF_LINKS];
  float Totalbits[NUMBER_OF_LINKS];

  int Hours[NUMBER_OF_QUADS];
  int Minutes[NUMBER_OF_QUADS];
  int Seconds[NUMBER_OF_QUADS];
  int temp;
  int SelectedChannel;


  int Locked[NUMBER_OF_LINKS];
  int ChannelOK[NUMBER_OF_LINKS];
  int WordAligned[NUMBER_OF_LINKS];
  int LaneAligned[NUMBER_OF_LINKS];    
  int Powerdown[NUMBER_OF_LINKS];
  int PLL_Locked[NUMBER_OF_LINKS];
  int Rx_FreqLocked[NUMBER_OF_LINKS];
  float ErrorCount[NUMBER_OF_LINKS]; 
  int ErrorCount_Reg[NUMBER_OF_LINKS];
  int tx_vodctrl[NUMBER_OF_LANES];
  int tx_preemp_0t[NUMBER_OF_LANES];
  int tx_preemp_1t[NUMBER_OF_LANES];
  int tx_preemp_2t[NUMBER_OF_LANES];
  int rx_eqdcgain[NUMBER_OF_LANES];
  int rx_eqctrl[NUMBER_OF_LANES];   
  int Quad;
  int i;
  int j;
  int BERInterval;
  int TimeInterval;
  int EyeQInterval;
  
 
  int Reg_data;
  int Ravail;

  int Serial_Loop[NUMBER_OF_LANES];
  int NOK;
  //int Loopback_All;
  
  float eye;
  int eye_phase;
  float BER_Minimum;
  int BER_Minimum_phase;
  int start_value;
  int end_value;
  int optimum_phase;
  int table_phase_start;
  int index;  
  
  //int eye1d;
  //int eyeqbw;
  int eyeq_status;
  int eyeq_control;
  int eyepolarity;
  
  float ErrorCount_Array[32][64]; 
  float BER_Array[32][64];
  
  float ErrorCount_Array_Bottom[32][64]; 
  float BER_Array_Bottom[32][64];
  
  int veye;
  int veye_top;
  int veye_bottom;
  int x,y;  
  
    int ErrorCount_Reg_L[NUMBER_OF_LANES];
  int ErrorCount_Reg_H[NUMBER_OF_LANES];
  int BitCount_L[NUMBER_OF_LANES];
  int BitCount_H[NUMBER_OF_LANES];
  
    int High_nibble;
  int Low_nibble;
  
  int Latency_Reg[0];
  int Temperature;
  int Error_Deskew[NUMBER_OF_QUADS];
  int Error_Decoder[NUMBER_OF_QUADS];
  int Latency_Maximum;
  int Latency_Minimum;
  int Latency_memory[HISTOGRAM_SIZE];

  
  
  //char line[100]; 
	
    
  /**********************************************************************/
  /*  Initialize PMA Settings                                     */
  /**********************************************************************/
  
  
  for (i = 0; i < NUMBER_OF_LANES ; i++)
  { 
  tx_vodctrl[i]    = VOD_800MV; 
  tx_preemp_0t[i]  = PRE_TAP_9N;
  tx_preemp_1t[i]  = POST_TAP1_20; 
  tx_preemp_2t[i]  = POST_TAP2_0;
  rx_eqdcgain[i]   = EQ_GAIN_1;
  rx_eqctrl[i]     = EQ_CTRL_0;
	  
  write_reconfiguration(i,VOD,tx_vodctrl[i]);
  write_reconfiguration(i,PRE_TAP,tx_preemp_0t[i]);  
  write_reconfiguration(i,POST_TAP1,tx_preemp_1t[i]);	  
  write_reconfiguration(i,POST_TAP2,tx_preemp_2t[i]);	
  write_reconfiguration(i,EQ_GAIN,rx_eqdcgain[i]);
  write_reconfiguration(i,EQ_CTRL,rx_eqctrl[i]);	 
  Serial_Loop[i]   = 0x1;	   //Enable Serial Loopback by default
  }
  
  SelectedChannel = 0; // Default Selected Channels is channel 0 
   
  
  Control_Reg[0] = 0x03FF;   // Enable serial loopback  


  
  /**********************************************************************/
  /*  Reset Design                                                   */
  /**********************************************************************/  
  
  for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {
        Control_Reg[i] = Control_Reg[i] | (0x8000);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        usleep(10);
        
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {      
        Control_Reg[i] = Control_Reg[i] & (0x7FFF);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        do
        {
          Counter_1ms_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
        } while (Counter_1ms_Reg[0] < 100);
    
 
   
   BERInterval = 2; // Set to 2 seconds as default;         
   EyeQInterval = 1; // Set to 1 ms as default.
		  
   Quad = 0;
   Powerdown[0] = 0;
   
  
  /**********************************************************************/
  /*  Main Program Loop                                                 */
  /**********************************************************************/  
  while (1)
	{
  
  Channel_Reg[0]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL_REG_BASE);

  
  RefClock_Reg[0]   = IORD_ALTERA_AVALON_PIO_DATA(BITRATE_REG_BASE); 
  DataClock_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(DATACLOCK_REG_BASE);
  
  Latency_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(LATENCY_REG_BASE);
    
  Refclockmultiplier[0] = (0x003F) & (Channel_Reg[0] >> 6);

  Bitrate_temp[0] =  ((float)RefClock_Reg[0]* Refclockmultiplier[0] * NUMBER_OF_LANES)/(1000000);
  Linerate_temp[0] =  ((float)RefClock_Reg[0]* Refclockmultiplier[0])/(1000000);
  Userdatarate_temp[0] = ((float)DataClock_Reg[0] * 64 * NUMBER_OF_LANES )/(1000000);
		
  Latency_measure_temp[0] = ((float)Latency_Reg[0] * 1000000000)/((float)DataClock_Reg[0]);
  
  Bitrate[0] = (Bitrate_temp[0]); // Convert the floats to integer numbers
  Linerate[0] = (Linerate_temp[0]); // Convert the floats to integer numbers
  Userdatarate[0] = (Userdatarate_temp[0]); // Convert the floats to integer numbers  
  Latency_measure[0] = (Latency_measure_temp[0]); // Convert the floats to integer numbers 
   
   
   
  Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);

  ErrorCount_Reg[0]   		 =  ((0x0000FFFF) & IORD_ALTERA_AVALON_PIO_DATA(BITERRORCOUNT_REG_BASE));	// Mask the upper 16 bits as they will also be read and could contain a value
  
  ErrorCount[0] = (float) (ErrorCount_Reg[0]);
  
  Temperature = IORD_ALTERA_AVALON_PIO_DATA(TEMPERATURE_REG_BASE);
  
//  Delay_Reg[0]	= IORD_ALTERA_AVALON_PIO_DATA(DELAY_REG_BASE);
//  Delay_Reg[1] = IORD_ALTERA_AVALON_PIO_DATA(DELAY_REG_2_BASE);
  
 
        
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {

  Locked[i]  = 0x0001 & (Channel_Reg[i] >> 15);    
  PLL_Locked[i]  = 0x0001 & (Channel_Reg[i] >> 14);
  LaneAligned[i] = 0x0001 & (Channel_Reg[i] >>  13);  
  Rx_FreqLocked[i]   = 0x0001 & (Channel_Reg[i] >> 12);  
  Error_Deskew[i] = 0x0001 & (Channel_Reg[i] >>  2);  
  Error_Decoder[i] = 0x0001 & (Channel_Reg[i] >>  1);	  
  WordAligned[i] = 0x0001 & (Channel_Reg[i] >>  0);    
  }
  
  
//   for (i = 0; i < NUMBER_OF_LANES ; i++)
//  {
//  Delay_lane[i]  = 0x0007 & (Delay_Reg[0] >> ((3*i)));       
//  } 

  
  
  Totalbits[0] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[0]*1000);
    
  for (i = 0; i < NUMBER_OF_QUADS ; i++)
  {
  Hours[i] =  Counter_1ms_Reg[i] / NUMBER_OF_MS_PER_HOUR ;
  Minutes[i] = (Counter_1ms_Reg[i] - (Hours[i] * NUMBER_OF_MS_PER_HOUR))/NUMBER_OF_MS_PER_MINUTE;
  Seconds[i] = (Counter_1ms_Reg[i] - (Hours[i] * NUMBER_OF_MS_PER_HOUR) - (Minutes[i] * NUMBER_OF_MS_PER_MINUTE)) / NUMBER_OF_MS_PER_SECOND;
  }
  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
      if (ErrorCount[i] > 0)
        BER[i] = ((ErrorCount[i])) / Totalbits[i];
      else
        //BER[i] = 0;
        BER[i] = ((float) 3) / Totalbits[i]; /* confidence level (CL) of 95% = -ln(1-CL) = 3*/
  } /* For Loop */  
   

    printf("\n\n\n");
    printf("Status Stratix V GX Synchronous Demo Design\n");
    printf("|-------------------------------------------------------------------|\n");
    printf("|Number of Lanes         : %2d  \n",NUMBER_OF_LANES);
    printf("|Line rate               : %4d Mbps \n",Linerate[0]);
    printf("|Aggregrate Line Rate    : %4d Mbps \n",Bitrate[0]);  
    printf("|Net User Data Bandwidth : %4d Mbps \n",Userdatarate[0]);  
    printf("|Measured Latency        : %4d ns \n",Latency_measure[0]);    
    printf("|SelectedChannel         : %2d  \n",SelectedChannel);  
    printf("|-------------------------------------------------------------------|\n");

    printf("\n");               
   
    printf("Status\n");
    printf("======\n");     

    printf("Lane            |");
    for (i = 0; i < NUMBER_OF_LANES ; i++)
    { 
      printf("%2X|",i);
    }
    printf("\n");
    printf("                |");
    for (i = 0; i < NUMBER_OF_LANES ; i++)
    { 
    printf("--|");
    }
    printf("\n");

    printf("Serial Loop     :");        
    for (i = 0; i < NUMBER_OF_LANES ; i++)
    {     
      printf("%2d|",Serial_Loop[i]);   
    }
    printf("\n"); 
	 
//    printf("Delay           :");        
//    for (i = 0; i < NUMBER_OF_LANES ; i++)
//    {     
//      printf("%2d|",Delay_lane[i]);   
//    }
//    printf("\n"); 	 
             
    printf("PLLLocked       :");        
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0) 
      printf("%2d",PLL_Locked[i]);
    else
      printf("  ");    
    }
    printf("\n"); 
    
    printf("FreqLocked      :");        
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",Rx_FreqLocked[i]);
    else
      printf("  ");    
    }
    printf("\n");

    printf("Error Deskew    :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",Error_Deskew[i]);
    else
      printf("  ");    
    }
    printf("\n"); 	 

    printf("Error Decoder   :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",Error_Decoder[i]);
    else
      printf("  ");    
    }
    printf("\n"); 	 
 
    printf("WordAligned     :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",WordAligned[i]);
    else
      printf("  ");    
    }
    printf("\n");     

    printf("LaneAligned     :");       
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)       
      printf("%2d",LaneAligned[i]);
    else
      printf("  ");    
    }
    printf("\n");         

      
    printf("DataLocked      :");

    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
    if (Powerdown[i] == 0)      
      printf("%2d",Locked[i]);
    else
      printf("  ");
    }
    printf("\n");
  
      
    printf("Errorcount      :");
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    {
        if ((Locked[i] == 0) || (Powerdown[i] == 1))
          printf("  ");
        else if (ErrorCount[i] > 99)
          printf("xx");
        else    
          printf("%2d",ErrorCount_Reg[i]);   
    }
    printf("\n\n");
   
 

    
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
      if (Powerdown[i] == 1)
        printf("Ch%2d Powered down\n",i);        
      else if (Locked[i] == 1)
      {
        if (ErrorCount[i] > 0)
              printf("BER Estimate Link    : %e    (# Errors/Totalbits)\n",BER[i]);
          else
              printf("BER (CL=0.95) Link   : %e\n",BER[i]);
      }
      else
        printf("No Data Lock achieved \n");
    }
    
    printf("\n");

 
                 

    printf("Test Time                  : %2dh %2dm %2ds\n",Hours[0],Minutes[0],Seconds[0]);             
    printf("Reference Clock Frequency  : %7d kHz\n",(RefClock_Reg[0])/1000); 
    printf("DataClock Frequency        : %7d kHZ\n",DataClock_Reg[0]/1000); 
    printf("Temperature StratixV GX    : %3d (Celcius)\n",Temperature);	 
    printf("\n");
    

       
    printf("\nSelect Action : \n");
    printf("===============\n");
    
    printf("1. Reset Link\n");
    printf("2. Force Re-Alignment on Rx Path\n");    
    printf("3. Show/Control Transceiver PMA Settings on Links\n");
    printf("4. Insert Biterrors on Link (%1d at a time)\n",(NUMBER_OF_LANES/2));
    printf("5. Reset ErrorCounter\n");
    printf("6. Show Status\n");
    printf("7. Control Serial loopback\n");    
    printf("8. Refresh BER every %d seconds \n",BERInterval);
    printf("9. Input new BER Time Interval\n");  
    printf("A. Select Channel for EyeQ Measurement\n");    
	 printf("G. Input new EyeQInterval time\n");	 
	 printf("M. Measure 2-D eye on selected channel using BERB Block\n");
    printf("T. Perform %4d resets and measure latency and delay\n",RESET_CYCLES);	 
    printf("0. Stop Test \n\n");
    printf("  Enter Choice :");
  
    rx_char = input_char(); 

    
    
		switch (rx_char)
		{
      case '1': /* Reset Link*/
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {
        Control_Reg[i] = Control_Reg[i] | (0x8000);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        usleep(10);
        
        
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {      
        Control_Reg[i] = Control_Reg[i] & (0x7FFF);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        do
        {
          Counter_1ms_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
        } while (Counter_1ms_Reg[0] < 1000);
        
        printf("Link Resetted.\n\n\n");
        break; //case '1'

      case '2': /*Force Re-Alignment on Rx Path */
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {
        Control_Reg[i] = Control_Reg[i] | (0x1000);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        usleep(10);
        
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {      
        Control_Reg[i] = Control_Reg[i] & (0xEFFF);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        usleep(10);
        
        printf("Force Re-Alignment on Rx Path.\n\n\n");
        break; //case '2'
                
               
	   case '3': /*Show/Control Transceiver Parameters */

        printf("3. Transceiver PMA Settings (all lanes have the same settings)");
           switch (tx_vodctrl[Quad])
           {
           case  VOD_200MV  : printf("    VOD Level           :  200mV\n"); break;
           case  VOD_400MV  : printf("    VOD Level           :  400mV\n"); break;
           case  VOD_600MV  : printf("    VOD Level           :  600mV\n"); break;
           case  VOD_700MV  : printf("    VOD Level           :  700mV\n"); break;           
           case  VOD_800MV  : printf("    VOD Level           :  800mV\n"); break;
           case  VOD_900MV  : printf("    VOD Level           :  900mV\n"); break;           
           case  VOD_1000MV : printf("    VOD Level           : 1000mV\n"); break;
           case  VOD_1200MV : printf("    VOD Level           : 1200mV\n"); break;                                                              
           default : break;
           }
   
           
           switch (tx_preemp_0t[Quad])
           {
           case  PRE_TAP_15N : printf("    Pre Tap Level       : -15\n"); break;
           case  PRE_TAP_14N : printf("    Pre Tap Level       : -14\n"); break;
           case  PRE_TAP_13N : printf("    Pre Tap Level       : -13\n"); break;
           case  PRE_TAP_12N : printf("    Pre Tap Level       : -12\n"); break;
           case  PRE_TAP_11N : printf("    Pre Tap Level       : -11\n"); break;
           case  PRE_TAP_10N : printf("    Pre Tap Level       : -10\n"); break;
           case  PRE_TAP_9N  : printf("    Pre Tap Level       : -9\n"); break;  
           case  PRE_TAP_8N  : printf("    Pre Tap Level       : -8\n"); break;              
           case  PRE_TAP_7N  : printf("    Pre Tap Level       : -7\n"); break;
           case  PRE_TAP_6N  : printf("    Pre Tap Level       : -6\n"); break;
           case  PRE_TAP_5N  : printf("    Pre Tap Level       : -5\n"); break;
           case  PRE_TAP_4N  : printf("    Pre Tap Level       : -4\n"); break;
           case  PRE_TAP_3N  : printf("    Pre Tap Level       : -3\n"); break;
           case  PRE_TAP_2N  : printf("    Pre Tap Level       : -2\n"); break;
           case  PRE_TAP_1N  : printf("    Pre Tap Level       : -1\n"); break;
           case  PRE_TAP_0   : printf("    Pre Tap Level       :  0\n"); break;
           case  PRE_TAP_1   : printf("    Pre Tap Level       :  1\n"); break;
           case  PRE_TAP_2   : printf("    Pre Tap Level       :  2\n"); break;
           case  PRE_TAP_3   : printf("    Pre Tap Level       :  3\n"); break;
           case  PRE_TAP_4   : printf("    Pre Tap Level       :  4\n"); break;
           case  PRE_TAP_5   : printf("    Pre Tap Level       :  5\n"); break;
           case  PRE_TAP_6   : printf("    Pre Tap Level       :  6\n"); break;
           case  PRE_TAP_7   : printf("    Pre Tap Level       :  7\n"); break; 
           case  PRE_TAP_8   : printf("    Pre Tap Level       :  8\n"); break;
           case  PRE_TAP_9   : printf("    Pre Tap Level       :  9\n"); break;
           case  PRE_TAP_10  : printf("    Pre Tap Level       :  10\n"); break;
           case  PRE_TAP_11  : printf("    Pre Tap Level       :  11\n"); break;
           case  PRE_TAP_12  : printf("    Pre Tap Level       :  12\n"); break;
           case  PRE_TAP_13  : printf("    Pre Tap Level       :  13\n"); break;
           case  PRE_TAP_14  : printf("    Pre Tap Level       :  14\n"); break;
           case  PRE_TAP_15  : printf("    Pre Tap Level       :  15\n"); break;               
           default : break;
           }

           switch (tx_preemp_1t[Quad])
           {
           case  POST_TAP1_0  : printf("    Post Tap 1 Level    :  0\n"); break;
           case  POST_TAP1_1  : printf("    Post Tap 1 Level    :  1\n"); break;    
           case  POST_TAP1_2  : printf("    Post Tap 1 Level    :  2\n"); break;
           case  POST_TAP1_3  : printf("    Post Tap 1 Level    :  3\n"); break;   
           case  POST_TAP1_4  : printf("    Post Tap 1 Level    :  4\n"); break;
           case  POST_TAP1_5  : printf("    Post Tap 1 Level    :  5\n"); break;   
           case  POST_TAP1_6  : printf("    Post Tap 1 Level    :  6\n"); break;
           case  POST_TAP1_7  : printf("    Post Tap 1 Level    :  7\n"); break;   
           case  POST_TAP1_8  : printf("    Post Tap 1 Level    :  8\n"); break;
           case  POST_TAP1_9  : printf("    Post Tap 1 Level    :  9\n"); break;   
           case  POST_TAP1_10 : printf("    Post Tap 1 Level    : 10\n"); break;
           case  POST_TAP1_11 : printf("    Post Tap 1 Level    : 11\n"); break;   
           case  POST_TAP1_12 : printf("    Post Tap 1 Level    : 12\n"); break;     
           case  POST_TAP1_13 : printf("    Post Tap 1 Level    : 13\n"); break;   
           case  POST_TAP1_14 : printf("    Post Tap 1 Level    : 14\n"); break;
           case  POST_TAP1_15 : printf("    Post Tap 1 Level    : 15\n"); break;   
           case  POST_TAP1_16 : printf("    Post Tap 1 Level    : 16\n"); break;
           case  POST_TAP1_17 : printf("    Post Tap 1 Level    : 17\n"); break;   
           case  POST_TAP1_18 : printf("    Post Tap 1 Level    : 18\n"); break;
           case  POST_TAP1_19 : printf("    Post Tap 1 Level    : 19\n"); break;   
           case  POST_TAP1_20 : printf("    Post Tap 1 Level    : 20\n"); break;
           case  POST_TAP1_21 : printf("    Post Tap 1 Level    : 21\n"); break;   
           case  POST_TAP1_22 : printf("    Post Tap 1 Level    : 22\n"); break; 
           case  POST_TAP1_23 : printf("    Post Tap 1 Level    : 23\n"); break;   
           case  POST_TAP1_24 : printf("    Post Tap 1 Level    : 24\n"); break;
           case  POST_TAP1_25 : printf("    Post Tap 1 Level    : 25\n"); break;   
           case  POST_TAP1_26 : printf("    Post Tap 1 Level    : 26\n"); break;
           case  POST_TAP1_27 : printf("    Post Tap 1 Level    : 27\n"); break;   
           case  POST_TAP1_28 : printf("    Post Tap 1 Level    : 28\n"); break;
           case  POST_TAP1_29 : printf("    Post Tap 1 Level    : 29\n"); break;   
           case  POST_TAP1_30 : printf("    Post Tap 1 Level    : 30\n"); break;
           case  POST_TAP1_31 : printf("    Post Tap 1 Level    : 31\n"); break;             
           default : break;
           }

           switch (tx_preemp_2t[Quad])
           {
           case  POST_TAP2_15N : printf("    Post Tap 2 Level       : -15\n"); break;
           case  POST_TAP2_14N : printf("    Post Tap 2 Level       : -14\n"); break;
           case  POST_TAP2_13N : printf("    Post Tap 2 Level       : -13\n"); break;
           case  POST_TAP2_12N : printf("    Post Tap 2 Level       : -12\n"); break;
           case  POST_TAP2_11N : printf("    Post Tap 2 Level       : -11\n"); break;
           case  POST_TAP2_10N : printf("    Post Tap 2 Level       : -10\n"); break;
           case  POST_TAP2_9N  : printf("    Post Tap 2 Level       : -9\n"); break;  
           case  POST_TAP2_8N  : printf("    Post Tap 2 Level       : -8\n"); break;              
           case  POST_TAP2_7N  : printf("    Post Tap 2 Level       : -7\n"); break;
           case  POST_TAP2_6N  : printf("    Post Tap 2 Level       : -6\n"); break;
           case  POST_TAP2_5N  : printf("    Post Tap 2 Level       : -5\n"); break;
           case  POST_TAP2_4N  : printf("    Post Tap 2 Level       : -4\n"); break;
           case  POST_TAP2_3N  : printf("    Post Tap 2 Level       : -3\n"); break;
           case  POST_TAP2_2N  : printf("    Post Tap 2 Level       : -2\n"); break;
           case  POST_TAP2_1N  : printf("    Post Tap 2 Level       : -1\n"); break;
           case  POST_TAP2_0   : printf("    Post Tap 2 Level       :  0\n"); break;
           case  POST_TAP2_1   : printf("    Post Tap 2 Level       :  1\n"); break;
           case  POST_TAP2_2   : printf("    Post Tap 2 Level       :  2\n"); break;
           case  POST_TAP2_3   : printf("    Post Tap 2 Level       :  3\n"); break;
           case  POST_TAP2_4   : printf("    Post Tap 2 Level       :  4\n"); break;
           case  POST_TAP2_5   : printf("    Post Tap 2 Level       :  5\n"); break;
           case  POST_TAP2_6   : printf("    Post Tap 2 Level       :  6\n"); break;
           case  POST_TAP2_7   : printf("    Post Tap 2 Level       :  7\n"); break; 
           case  POST_TAP2_8   : printf("    Post Tap 2 Level       :  8\n"); break;
           case  POST_TAP2_9   : printf("    Post Tap 2 Level       :  9\n"); break;
           case  POST_TAP2_10  : printf("    Post Tap 2 Level       :  10\n"); break;
           case  POST_TAP2_11  : printf("    Post Tap 2 Level       :  11\n"); break;
           case  POST_TAP2_12  : printf("    Post Tap 2 Level       :  12\n"); break;
           case  POST_TAP2_13  : printf("    Post Tap 2 Level       :  13\n"); break;
           case  POST_TAP2_14  : printf("    Post Tap 2 Level       :  14\n"); break;
                                                                                                                                                                                                           
           default : break;
           }

           switch (rx_eqdcgain[Quad])
           {
           case  EQ_GAIN_0    : printf("    Equalizer Gain Level    :  0\n"); break;
           case  EQ_GAIN_1    : printf("    Equalizer Gain Level    :  1\n"); break;
           case  EQ_GAIN_2    : printf("    Equalizer Gain Level    :  2\n"); break;   
           case  EQ_GAIN_3    : printf("    Equalizer Gain Level    :  3\n"); break;
           case  EQ_GAIN_4    : printf("    Equalizer Gain Level    :  4\n"); break;
           case  EQ_GAIN_5    : printf("    Equalizer Gain Level    :  5\n"); break;   
           case  EQ_GAIN_6    : printf("    Equalizer Gain Level    :  6\n"); break;
           case  EQ_GAIN_7    : printf("    Equalizer Gain Level    :  7\n"); break;
				  
           default : break;
           }

           switch (rx_eqctrl[Quad])
           {
           case  EQ_CTRL_0    : printf("    Equalizer Control Level :  0\n"); break;
           case  EQ_CTRL_1    : printf("    Equalizer Control Level :  1\n"); break;
           case  EQ_CTRL_2    : printf("    Equalizer Control Level :  2\n"); break;
           case  EQ_CTRL_3    : printf("    Equalizer Control Level :  3\n"); break;
           case  EQ_CTRL_4    : printf("    Equalizer Control Level :  4\n"); break;
           case  EQ_CTRL_5    : printf("    Equalizer Control Level :  5\n"); break;
           case  EQ_CTRL_6    : printf("    Equalizer Control Level :  6\n"); break;
           case  EQ_CTRL_7    : printf("    Equalizer Control Level :  7\n"); break;
           case  EQ_CTRL_8    : printf("    Equalizer Control Level :  8\n"); break;
           case  EQ_CTRL_9    : printf("    Equalizer Control Level :  9\n"); break;
           case  EQ_CTRL_10   : printf("    Equalizer Control Level : 10\n"); break;
           case  EQ_CTRL_11   : printf("    Equalizer Control Level : 11\n"); break;
           case  EQ_CTRL_12   : printf("    Equalizer Control Level : 12\n"); break;
           case  EQ_CTRL_13   : printf("    Equalizer Control Level : 13\n"); break;
           case  EQ_CTRL_14   : printf("    Equalizer Control Level : 14\n"); break;   
           case  EQ_CTRL_15   : printf("    Equalizer Control Level : 15\n"); break;                                                                                                                                                                                                                                                                                    
           default : break;
           }

        do {
        printf("\n");  
        
        printf("    New VOD Level Channel %1d               \n",Quad);
        printf("    ======================               \n");
        printf("    For VOD Level =  200 mV choose '0' \n");
        printf("    For VOD Level =  400 mV choose '1' \n");
        printf("    For VOD Level =  600 mV choose '2' \n");
        printf("    For VOD Level =  700 mV choose '3' \n");        
        printf("    For VOD Level =  800 mV choose '4' \n");
        printf("    For VOD Level =  900 mV choose '5' \n");        
        printf("    For VOD Level = 1000 mV choose '6' \n");
        printf("    For VOD Level = 1200 mV choose '7' \n");
        printf("    No Change               choose 'X' \n\n");    
        printf("    Choice :");      
        
        temp = 0xffffffff;
                                             
        rx_char = input_char(); 
        switch (rx_char)
        {
        case '0': temp =  VOD_200MV; break;
        case '1': temp =  VOD_400MV; break;
        case '2': temp =  VOD_600MV; break;
        case '3': temp =  VOD_700MV; break;        
        case '4': temp =  VOD_800MV; break;
        case '5': temp =  VOD_900MV; break;        
        case '6': temp =  VOD_1000MV; break;
        case '7': temp =  VOD_1200MV; break;
        case 'x': 
        case 'X': temp =  tx_vodctrl[Quad];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        tx_vodctrl[Quad] = temp; 
        
        printf("\n");  
        
        do {
        printf("    New Pre Tap Level Channel %1d               \n",Quad);
        printf("    =========================                \n");
          printf("    For Pre Tap =  -15 choose '1' \n");
          printf("    For Pre Tap =  -14 choose '2' \n");
          printf("    For Pre Tap =  -13 choose '3' \n");
          printf("    For Pre Tap =  -12 choose '4' \n");
          printf("    For Pre Tap =  -11 choose '5' \n");
          printf("    For Pre Tap =  -10 choose '6' \n");
          printf("    For Pre Tap =  -9  choose '7' \n");
          printf("    For Pre Tap =  -8  choose '8' \n");
          printf("    For Pre Tap =  -7  choose '9' \n"); 
          printf("    For Pre Tap =  -6  choose 'A' \n");          
          printf("    For Pre Tap =  -5  choose 'B' \n");
          printf("    For Pre Tap =  -4  choose 'C' \n");
          printf("    For Pre Tap =  -3  choose 'D' \n");
          printf("    For Pre Tap =  -2  choose 'E' \n");          
          printf("    For Pre Tap =  -1  choose 'F' \n");
          printf("    For Pre Tap =   0  choose '0' \n");
          printf("    For Pre Tap =   1  choose 'G' \n");        
          printf("    For Pre Tap =   2  choose 'H' \n");
          printf("    For Pre Tap =   3  choose 'I' \n");
          printf("    For Pre Tap =   4  choose 'J' \n");
          printf("    For Pre Tap =   5  choose 'K' \n");        
          printf("    For Pre Tap =   6  choose 'L' \n");
          printf("    For Pre Tap =   7  choose 'M' \n");
          printf("    For Pre Tap =   8  choose 'N' \n");
          printf("    For Pre Tap =   9  choose 'O' \n");
          printf("    For Pre Tap =  10  choose 'P' \n");
          printf("    For Pre Tap =  11  choose 'Q' \n");        
          printf("    For Pre Tap =  12  choose 'R' \n");
          printf("    For Pre Tap =  13  choose 'S' \n");  
          printf("    For Pre Tap =  14  choose 'T' \n");
          printf("    For Pre Tap =  15  choose 'U' \n");                                                                                                        
        printf("    No Change         choose 'X' \n\n");    
        printf("    Choice :");
        
        temp = 0xffffffff;
                                                   
        rx_char = input_char(); 
        switch (rx_char)
        {
        case '0': temp =  PRE_TAP_0; break;
        case '1': temp =  PRE_TAP_15N; break;
        case '2': temp =  PRE_TAP_14N; break;
        case '3': temp =  PRE_TAP_13N; break;
        case '4': temp =  PRE_TAP_12N; break;
        case '5': temp =  PRE_TAP_11N; break;
        case '6': temp =  PRE_TAP_10N; break;
        case '7': temp =  PRE_TAP_9N; break;
        case '8': temp =  PRE_TAP_8N; break;
        case '9': temp =  PRE_TAP_7N; break;
        case 'a':
        case 'A': temp =  PRE_TAP_6N; break;
        case 'b':
        case 'B': temp =  PRE_TAP_5N; break;
        case 'c':
        case 'C': temp =  PRE_TAP_4N; break;
        case 'd':
        case 'D': temp =  PRE_TAP_3N; break;
        case 'e':
        case 'E': temp =  PRE_TAP_2N; break;      
        case 'f':
        case 'F': temp =  PRE_TAP_1N; break;   
        case 'g':
        case 'G': temp =  PRE_TAP_1; break;  
        case 'h':
        case 'H': temp =  PRE_TAP_2; break;  
        case 'i':
        case 'I': temp =  PRE_TAP_3; break;  
        case 'j':
        case 'J': temp =  PRE_TAP_4; break;   
        case 'k':
        case 'K': temp =  PRE_TAP_5; break;  
        case 'l':
        case 'L': temp =  PRE_TAP_6; break;  
        case 'm':
        case 'M': temp =  PRE_TAP_7; break;  
        case 'n':
        case 'N': temp =  PRE_TAP_8; break;   
        case 'o':
        case 'O': temp =  PRE_TAP_9; break;  
        case 'p':
        case 'P': temp =  PRE_TAP_10; break;  
        case 'q':
        case 'Q': temp =  PRE_TAP_11; break;  
        case 'r':
        case 'R': temp =  PRE_TAP_12; break;   
        case 's':
        case 'S': temp =  PRE_TAP_13; break;  
        case 't':
        case 'T': temp =  PRE_TAP_14; break;  
        case 'u':
        case 'U': temp =  PRE_TAP_15; break;           
        case 'x': 
        case 'X': temp = tx_preemp_0t[Quad];break;
        default : break;
        }
        } while (temp == 0xffffffff);       
        
        tx_preemp_0t[Quad] = temp;
       

        printf("\n");  
        
        do {
        printf("    New Post Tap 1 Level Channel %1d               \n",Quad);
        printf("    =========================                \n");
          printf("    For Post Tap 1 =  0 choose '0' \n");        
          printf("    For Post Tap 1 =  1 choose '1' \n"); 
          printf("    For Post Tap 1 =  2 choose '2' \n"); 
          printf("    For Post Tap 1 =  3 choose '3' \n"); 
          printf("    For Post Tap 1 =  4 choose '4' \n");
          printf("    For Post Tap 1 =  5 choose '5' \n");
          printf("    For Post Tap 1 =  6 choose '6' \n");
          printf("    For Post Tap 1 =  7 choose '7' \n");
          printf("    For Post Tap 1 =  8 choose '8' \n");
          printf("    For Post Tap 1 =  9 choose '9' \n");
          printf("    For Post Tap 1 = 10 choose 'A' \n");          
          printf("    For Post Tap 1 = 11 choose 'B' \n");
          printf("    For Post Tap 1 = 12 choose 'C' \n");
          printf("    For Post Tap 1 = 13 choose 'D' \n");
          printf("    For Post Tap 1 = 14 choose 'E' \n");
          printf("    For Post Tap 1 = 15 choose 'F' \n");
          printf("    For Post Tap 1 = 16 choose 'G' \n");
          printf("    For Post Tap 1 = 17 choose 'H' \n");
          printf("    For Post Tap 1 = 18 choose 'I' \n");
          printf("    For Post Tap 1 = 19 choose 'J' \n");
          printf("    For Post Tap 1 = 20 choose 'K' \n");
          printf("    For Post Tap 1 = 21 choose 'L' \n"); 
          printf("    For Post Tap 1 = 22 choose 'M' \n");
          printf("    For Post Tap 1 = 23 choose 'N' \n");
          printf("    For Post Tap 1 = 24 choose 'O' \n");
          printf("    For Post Tap 1 = 25 choose 'P' \n");              
          printf("    For Post Tap 1 = 26 choose 'Q' \n");
          printf("    For Post Tap 1 = 27 choose 'R' \n");
          printf("    For Post Tap 1 = 28 choose 'S' \n");
          printf("    For Post Tap 1 = 29 choose 'T' \n");
          printf("    For Post Tap 1 = 30 choose 'U' \n");
          printf("    For Post Tap 1 = 31 choose 'V' \n");                                                                                                         
          printf("    No Change           choose 'X' \n\n");    
          printf("    Choice :");
        
        temp = 0xffffffff;
                                        
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  POST_TAP1_0; break;
        case '1': temp =  POST_TAP1_1; break;
        case '2': temp =  POST_TAP1_2; break;
        case '3': temp =  POST_TAP1_3; break;
        case '4': temp =  POST_TAP1_4; break;
        case '5': temp =  POST_TAP1_5; break;
        case '6': temp =  POST_TAP1_6; break;
        case '7': temp =  POST_TAP1_7; break;
        case '8': temp =  POST_TAP1_8; break;
        case '9': temp =  POST_TAP1_9; break;
        case 'a':
        case 'A': temp =  POST_TAP1_10; break;
        case 'b':
        case 'B': temp =  POST_TAP1_11; break;
        case 'c':
        case 'C': temp =  POST_TAP1_12; break;  
        case 'd':        
        case 'D': temp =  POST_TAP1_13; break;
        case 'e':        
        case 'E': temp =  POST_TAP1_14; break;
        case 'f':        
        case 'F': temp =  POST_TAP1_15; break;
        case 'g':
        case 'G': temp =  POST_TAP1_16; break;
        case 'h':
        case 'H': temp =  POST_TAP1_17; break;
        case 'i':
        case 'I': temp =  POST_TAP1_18; break;
        case 'j':
        case 'J': temp =  POST_TAP1_19; break;
        case 'k':
        case 'K': temp =  POST_TAP1_20; break;  
        case 'l':
        case 'L': temp =  POST_TAP1_21; break;
        case 'm':
        case 'M': temp =  POST_TAP1_22; break;  
        case 'n':        
        case 'N': temp =  POST_TAP1_23; break;
        case 'o':        
        case 'O': temp =  POST_TAP1_24; break;
        case 'p':        
        case 'P': temp =  POST_TAP1_25; break;
        case 'q':
        case 'Q': temp =  POST_TAP1_26; break;
        case 'r':
        case 'R': temp =  POST_TAP1_27; break;
        case 's':
        case 'S': temp =  POST_TAP1_28; break;
        case 't':
        case 'T': temp =  POST_TAP1_29; break;
        case 'u':
        case 'U': temp =  POST_TAP1_30; break;  
        case 'v':
        case 'V': temp =  POST_TAP1_31; break;          
        case 'x':         
        case 'X': temp = tx_preemp_1t[Quad];break;
        default : break;
        }
        } while (temp == 0xffffffff);           
        
        
        tx_preemp_1t[Quad] = temp;        
           
        printf("\n");  
        
        do {
        printf("    New Post Tap 2 Level Channel %1d               \n",Quad);
        printf("    =========================                \n");
          printf("    For Post Tap 2 =  -15 choose '1' \n");
          printf("    For Post Tap 2 =  -14 choose '2' \n");
          printf("    For Post Tap 2 =  -13 choose '3' \n");
          printf("    For Post Tap 2 =  -12 choose '4' \n");
          printf("    For Post Tap 2 =  -11 choose '5' \n");
          printf("    For Post Tap 2 =  -10 choose '6' \n");
          printf("    For Post Tap 2 =  -9  choose '7' \n");
          printf("    For Post Tap 2 =  -8  choose '8' \n");
          printf("    For Post Tap 2 =  -7  choose '9' \n"); 
          printf("    For Post Tap 2 =  -6  choose 'A' \n");          
          printf("    For Post Tap 2 =  -5  choose 'B' \n");
          printf("    For Post Tap 2 =  -4  choose 'C' \n");
          printf("    For Post Tap 2 =  -3  choose 'D' \n");
          printf("    For Post Tap 2 =  -2  choose 'E' \n");          
          printf("    For Post Tap 2 =  -1  choose 'F' \n");
          printf("    For Post Tap 2 =   0  choose '0' \n");
          printf("    For Post Tap 2 =   1  choose 'G' \n");    
          printf("    For Post Tap 2 =   2  choose 'H' \n");
          printf("    For Post Tap 2 =   3  choose 'I' \n");
          printf("    For Post Tap 2 =   4  choose 'J' \n");
          printf("    For Post Tap 2 =   5  choose 'K' \n");        
          printf("    For Post Tap 2 =   6  choose 'L' \n");
          printf("    For Post Tap 2 =   7  choose 'M' \n");
          printf("    For Post Tap 2 =   8  choose 'N' \n");
          printf("    For Post Tap 2 =   9  choose 'O' \n");
          printf("    For Post Tap 2 =  10  choose 'P' \n");
          printf("    For Post Tap 2 =  11  choose 'Q' \n");        
          printf("    For Post Tap 2 =  12  choose 'R' \n");
          printf("    For Post Tap 2 =  13  choose 'S' \n");  
          printf("    For Post Tap 2 =  14  choose 'T' \n");
          printf("    For Post Tap 2 =  15  choose 'U' \n");                                                                                                    
        printf("    No Change         choose 'X' \n\n");    
        printf("    Choice :");
        
        temp = 0xffffffff;
                                                   
        rx_char = input_char(); 
        switch (rx_char)
        {
        case '0': temp =  POST_TAP2_0; break;
        case '1': temp =  POST_TAP2_15N; break;
        case '2': temp =  POST_TAP2_14N; break;
        case '3': temp =  POST_TAP2_13N; break;
        case '4': temp =  POST_TAP2_12N; break;
        case '5': temp =  POST_TAP2_11N; break;
        case '6': temp =  POST_TAP2_10N; break;
        case '7': temp =  POST_TAP2_9N; break;
        case '8': temp =  POST_TAP2_8N; break;
        case '9': temp =  POST_TAP2_7N; break;
        case 'a':
        case 'A': temp =  POST_TAP2_6N; break;
        case 'b':
        case 'B': temp =  POST_TAP2_5N; break;
        case 'c':
        case 'C': temp =  POST_TAP2_4N; break;
        case 'd':
        case 'D': temp =  POST_TAP2_3N; break;
        case 'e':
        case 'E': temp =  POST_TAP2_2N; break;      
        case 'f':
        case 'F': temp =  POST_TAP2_1N; break;   
        case 'g':
        case 'G': temp =  POST_TAP2_1; break;  
        case 'h':
        case 'H': temp =  POST_TAP2_2; break;  
        case 'i':
        case 'I': temp =  POST_TAP2_3; break;  
        case 'j':
        case 'J': temp =  POST_TAP2_4; break;   
        case 'k':
        case 'K': temp =  POST_TAP2_5; break;  
        case 'l':
        case 'L': temp =  POST_TAP2_6; break;  
        case 'm':
        case 'M': temp =  POST_TAP2_7; break;  
        case 'n':
        case 'N': temp =  POST_TAP2_8; break;   
        case 'o':
        case 'O': temp =  POST_TAP2_9; break;  
        case 'p':
        case 'P': temp =  POST_TAP2_10; break;  
        case 'q':
        case 'Q': temp =  POST_TAP2_11; break;  
        case 'r':
        case 'R': temp =  POST_TAP2_12; break;   
        case 's':
        case 'S': temp =  POST_TAP2_13; break;  
        case 't':
        case 'T': temp =  POST_TAP2_14; break;  
        case 'u':
        case 'U': temp =  POST_TAP2_15; break;           
        case 'x': 
        case 'X': temp = tx_preemp_2t[Quad];break;
        default : break;
        }
        } while (temp == 0xffffffff);     
        
        tx_preemp_2t[Quad] = temp;        
        

       do {
        printf("\n");  
        
        printf("    New Equalizer Level Channel %1d               \n",Quad);
        printf("    ============================                \n");
        printf("    For Equalizer Level  =  0 choose '0' \n");
        printf("    For Equalizer Level  =  1 choose '1' \n");
        printf("    For Equalizer Level  =  2 choose '2' \n");
        printf("    For Equalizer Level  =  3 choose '3' \n");
        printf("    For Equalizer Level  =  4 choose '4' \n");
        printf("    For Equalizer Level  =  5 choose '5' \n");
        printf("    For Equalizer Level  =  6 choose '6' \n");
        printf("    For Equalizer Level  =  7 choose '7' \n");  
        printf("    For Equalizer Level  =  8 choose '8' \n");
        printf("    For Equalizer Level  =  9 choose '9' \n");
        printf("    For Equalizer Level  = 10 choose 'A' \n");
        printf("    For Equalizer Level  = 11 choose 'B' \n");
        printf("    For Equalizer Level  = 12 choose 'C' \n");
        printf("    For Equalizer Level  = 13 choose 'D' \n");
        printf("    For Equalizer Level  = 14 choose 'E' \n");
        printf("    For Equalizer Level  = 15 choose 'F' \n");                        
        printf("    No Change                 choose 'X' \n\n");    
        printf("    Choice :");      
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  EQ_CTRL_0; break;
        case '1': temp =  EQ_CTRL_1; break;
        case '2': temp =  EQ_CTRL_2; break;
        case '3': temp =  EQ_CTRL_3; break;
        case '4': temp =  EQ_CTRL_4; break;
        case '5': temp =  EQ_CTRL_5; break;
        case '6': temp =  EQ_CTRL_6; break;
        case '7': temp =  EQ_CTRL_7; break;
        case '8': temp =  EQ_CTRL_8; break;
        case '9': temp =  EQ_CTRL_9; break;
        case 'a':
        case 'A': temp =  EQ_CTRL_10; break;
        case 'b':
        case 'B': temp =  EQ_CTRL_11; break;
        case 'c':
        case 'C': temp =  EQ_CTRL_12; break;
        case 'd':
        case 'D': temp =  EQ_CTRL_13; break;
        case 'e':
        case 'E': temp =  EQ_CTRL_14; break;
        case 'f':
        case 'F': temp =  EQ_CTRL_15; break;                        
        case 'x': 
        case 'X': temp =  rx_eqctrl[Quad];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        rx_eqctrl[Quad] = temp; 
                

       do {
        printf("\n");  
        
        printf("    New Equalizer DC Gain Channel %1d               \n",Quad);
        printf("    =============================                \n");
        printf("    For Equalizer DC Gain  =  0 choose '0' \n");
        printf("    For Equalizer DC Gain  =  1 choose '1' \n");
        printf("    For Equalizer DC Gain  =  2 choose '2' \n");     
        printf("    For Equalizer DC Gain  =  3 choose '3' \n");
        printf("    For Equalizer DC Gain  =  4 choose '4' \n");
        printf("    For Equalizer DC Gain  =  5 choose '5' \n");     
        printf("    For Equalizer DC Gain  =  6 choose '6' \n");
        printf("    For Equalizer DC Gain  =  7 choose '7' \n"); 			 
        printf("    No Change                   choose 'X' \n\n");    
        printf("    Choice :");      
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  EQ_GAIN_0; break;
        case '1': temp =  EQ_GAIN_1; break;
        case '2': temp =  EQ_GAIN_2; break;
        case '3': temp =  EQ_GAIN_3; break;
        case '4': temp =  EQ_GAIN_4; break; 
        case '5': temp =  EQ_GAIN_5; break;
        case '6': temp =  EQ_GAIN_6; break;
        case '7': temp =  EQ_GAIN_7; break;             
        case 'x': 
        case 'X': temp =  rx_eqdcgain[Quad];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        rx_eqdcgain[Quad] = temp; 
        

        // Debug Purposes
         printf("    tx_vodctrl = %d                \n",tx_vodctrl[Quad]);
         printf("    tx_preemp_0t = %d              \n",tx_preemp_0t[Quad]);
         printf("    tx_preemp_1t = %d              \n",tx_preemp_1t[Quad]);
         printf("    tx_preemp_2t = %d              \n",tx_preemp_2t[Quad]);
         printf("    rx_eqdcgain  = %d              \n",rx_eqdcgain[Quad] );
         printf("    rx_eqctrl  = %d              \n",rx_eqctrl[Quad] );

			for (i = 0; i < NUMBER_OF_LANES ; i++)
			{ 
		   write_reconfiguration(i,VOD,tx_vodctrl[Quad]);
			write_reconfiguration(i,PRE_TAP,tx_preemp_0t[Quad]);
			write_reconfiguration(i,POST_TAP1,tx_preemp_1t[Quad]);	  
			write_reconfiguration(i,POST_TAP2,tx_preemp_2t[Quad]);	
			write_reconfiguration(i,EQ_GAIN,rx_eqdcgain[Quad]);	 
			write_reconfiguration(i,EQ_CTRL,rx_eqctrl[Quad]);	 
			}  
                      
				printf("\nTransceiver PMA parameters of all lanes updated\n\n");
				
				break; //case '3'
        
		case '4': /* Insert Biterrors */
     
        printf("4. Insert Biterror on Link\n\n\n");

        switch(Quad)
        {
        case 0 : 
        Control_Reg[0] = Control_Reg[0] | (0x4000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        Control_Reg[0] = Control_Reg[0] & (0xBFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        break;
       
    
        default : break;
        }
         
		break; /* Case 4 */
	
		case '5': /* Reset ErrorCounter */
        
	      printf("5. Reset ErrorCounter \n\n\n");
   
        
        switch(Quad)
        {
        case 0 : 
        Control_Reg[0] = Control_Reg[0] | (0x2000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        Control_Reg[0] = Control_Reg[0] & (0xDFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        break;
        
    
        default : break;
        }
        
        usleep(100000);     
            
       break;
	
      case '6': /* Show Status */ 
            
       break;        

      case '7': /* Control Serial Loopback */ 
            
		
    	printf("\nCurrent Status of Serial Loopback :\n");
    

		printf("Lane              |");
		for (i = 0; i < NUMBER_OF_LANES ; i++)
		{ 
		  printf("%2X|",i);
		}
		printf("\n");
		printf("                  |");
		for (i = 0; i < NUMBER_OF_LANES ; i++)
		{ 
		printf("--|");
		}
		printf("\n");
	
		printf("Serial Loop       :");        
		for (i = 0; i < NUMBER_OF_LANES ; i++)
		{     
		  printf("%2d|",Serial_Loop[i]);   
		}
		printf("\n"); 
  
              do {
               printf("\n");  
               
               printf("    Toggling Serial Loopback               \n");
               printf("    =========================              \n");
               printf("    Toggle Loopback on Lane  0 (GXB_TXLp_12/GXB_RXLp_12) choose '0' \n");
               printf("    Toggle Loopback on Lane  1 (GXB_TXLp_13/GXB_RXLp_13) choose '1' \n");
               printf("    Toggle Loopback on Lane  2 (GXB_TXLp_14/GXB_RXLp_14) choose '2' \n");
               printf("    Toggle Loopback on Lane  3 (GXB_TXLp_15/GXB_RXLp_15) choose '3' \n");
               printf("    Toggle Loopback on Lane  4 (GXB_TXLp_16/GXB_RXLp_16) choose '4' \n");					  
               printf("    Toggle Loopback on Lane  5 (GXB_TXLp_17/GXB_RXLp_17) choose '5' \n");	
               printf("    Toggle Loopback on Lane  6 (GXB_TXLp_18/GXB_RXLp_18) choose '6' \n");	
               printf("    Toggle Loopback on Lane  7 (GXB_TXLp_19/GXB_RXLp_19) choose '7' \n");	
               printf("    Toggle Loopback on Lane  8 (GXB_TXLp_20/GXB_RXLp_20) choose '8' \n");						  
               printf("    Toggle Loopback on Lane  9 (GXB_TXLp_21/GXB_RXLp_21) choose '9' \n");						  
					printf("    Enable Loopback on all Lanes choose 'E' \n");
               printf("    Disable Loopback on all Lanes choose 'D' \n");                    
               printf("    No Change                 choose 'X' \n\n");    
               printf("    Choice :");      
               
               temp = 0xffffffff;
                                                    
               rx_char   = input_char();
               switch (rx_char)
               {
               case '0': temp =  0; 
							if (Serial_Loop[0] == 1)
								Serial_Loop[0] = 0;
							else
								Serial_Loop[0] = 1;
						 break;
               case '1': temp =  1; 
							if (Serial_Loop[1] == 1)
								Serial_Loop[1] = 0;
							else
								Serial_Loop[1] = 1;
						 break;
               case '2': temp =  2; 
							if (Serial_Loop[2] == 1)
								Serial_Loop[2] = 0;
							else
								Serial_Loop[2] = 1;
						 break;
               case '3': temp =  3; 
							if (Serial_Loop[3] == 1)
								Serial_Loop[3] = 0;
							else
								Serial_Loop[3] = 1;
						 break;
               case '4': temp =  4; 
							if (Serial_Loop[4] == 1)
								Serial_Loop[4] = 0;
							else
								Serial_Loop[4] = 1;
						 break;	
               case '5': temp =  5; 
							if (Serial_Loop[5] == 1)
								Serial_Loop[5] = 0;
							else
								Serial_Loop[5] = 1;
						 break;							
               case '6': temp =  6; 
							if (Serial_Loop[6] == 1)
								Serial_Loop[6] = 0;
							else
								Serial_Loop[6] = 1;
						 break;
               case '7': temp =  7; 
							if (Serial_Loop[7] == 1)
								Serial_Loop[7] = 0;
							else
								Serial_Loop[7] = 1;
						 break;
               case '8': temp =  8; 
							if (Serial_Loop[8] == 1)
								Serial_Loop[8] = 0;
							else
								Serial_Loop[8] = 1;
						 break;
               case '9': temp =  9; 
							if (Serial_Loop[9] == 1)
								Serial_Loop[9] = 0;
							else
								Serial_Loop[9] = 1;
						 break;					
               case 'e':
               case 'E': temp =  14; 
						 for (i = 0; i < NUMBER_OF_LANES ; i++)
							{ 
							Serial_Loop[i] = 1;
							}
						 break;

               case 'd':
               case 'D': temp =  13; 
						 for (i = 0; i < NUMBER_OF_LANES ; i++)
							{ 
							Serial_Loop[i] = 0;
							}
						 break;
																	                        
               case 'x': 
               case 'X': temp =  128 ;break;
               default : break;
               }
               } while (temp == 0xffffffff);        	
   
    	Control_Reg[0]   = (Control_Reg[0] & (0xFFFE)) | (Serial_Loop[0]);
    	Control_Reg[0] 	= (Control_Reg[0] & (0xFFFD)) | (Serial_Loop[1] << 1);   
    	Control_Reg[0] 	= (Control_Reg[0] & (0xFFFB)) | (Serial_Loop[2] << 2);
    	Control_Reg[0] 	= (Control_Reg[0] & (0xFFF7)) | (Serial_Loop[3] << 3);
		Control_Reg[0] 	= (Control_Reg[0] & (0xFFEF)) | (Serial_Loop[4] << 4);
     	Control_Reg[0] 	= (Control_Reg[0] & (0xFFDF)) | (Serial_Loop[5] << 5);   
    	Control_Reg[0] 	= (Control_Reg[0] & (0xFFBF)) | (Serial_Loop[6] << 6);
    	Control_Reg[0] 	= (Control_Reg[0] & (0xFF7F)) | (Serial_Loop[7] << 7); 
		Control_Reg[0] 	= (Control_Reg[0] & (0xFEFF)) | (Serial_Loop[8] << 8);
     	Control_Reg[0] 	= (Control_Reg[0] & (0xFDFF)) | (Serial_Loop[9] << 9);					
    	
        

        Control_Reg[0] = Control_Reg[0] | (0x8000);

        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        usleep(10);
        
        
        Control_Reg[0] = Control_Reg[0] & (0x7FFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        do
        {
          Counter_1ms_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
        } while (Counter_1ms_Reg[0] < 1000);
        
        printf("Changed serial loop and Link Resetted.\n\n\n");
        break; //case '7'        



     
     case '8': //continuously update BER and errorcount every BERInterval seconds

        printf("\n");  
        printf("Show BER + ErrorCount every %d seconds              \n",BERInterval);
        printf("Press any key + Enter to stop the loop\n");
        printf("=========================================               \n");
        
        temp = 0;
        
        Reg_data = IORD_ALTERA_AVALON_JTAG_UART_DATA(ALT_STDIN_BASE);
        // Shift rigth of 16 to get access to the number of characters remaining in the read fifo
        Ravail = Reg_data >> 16;
        
        //printf("\nRavail %d :\n",Ravail);
        
        do
          {
        
        // Reset ErrorCount on Selected Channel
            switch(Quad)
            {
            case 0 : 
            Control_Reg[0] = Control_Reg[0] | (0x2000);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
            
            Control_Reg[0] = Control_Reg[0] & (0xDFFF);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
            break;
            
        
            default : break;
            }// end reset  
            
        TimeInterval = BERInterval * 1000;
        
        switch(Quad)
            {
             case 0 :
                do
                {
                Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
                } while (Counter_1ms_Reg[0] < TimeInterval);
             break;  
             default : break;
            } // End Case                        
             
       
        
            //Read ErrorCount registers
            
              Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
 
					ErrorCount_Reg[0]   		 =  ((0x0000FFFF) & IORD_ALTERA_AVALON_PIO_DATA(BITERRORCOUNT_REG_BASE));


                             
   
           for (i = 0; i < NUMBER_OF_LINKS ; i++)
              {
                ErrorCount[i] = (float) (ErrorCount_Reg[i]);
              }

              Totalbits[0] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[0]*1000);
  
            for (i = 0; i < NUMBER_OF_LINKS ; i++)
            {
                if (ErrorCount[i] > 0)
                  BER[i] = ((ErrorCount[i])) / Totalbits[i];
                else
                  BER[i] = 0; 
            } /* For Loop */   
            

      
            if (Locked[0] == 1)
                {
                if (ErrorCount[0] > 0)
                        printf("Link : BER Estimate : %e    ErrorCount  : %e \n",BER[0],ErrorCount[0]);
                    else
                        printf("Link : BER          : 0\n");
                }
            else
              printf("No Data Lock on Link \n");
         
                
           
        temp = temp + 1;
                                           
 
        //} while (temp <= 1000); // loop a 1000 times
        Reg_data = IORD_ALTERA_AVALON_JTAG_UART_DATA(ALT_STDIN_BASE);
        Ravail = Reg_data >> 16;
        } while(!Ravail);
        rx_char   = input_char();
        
        printf("\n Stopped\n");
        
        break; // Case '8'

      case '9': /* New BER Time Interval */
        
        printf("9. Provide BER Time Interval\n");
        printf("   Current BER Time Interval   : %2d seconds\n", BERInterval);
        printf("   Insert new BER Interval (Hexadecimal Number from 01 to FF : ");    
        
        BERInterval = input_byte();
              
        printf("\n    New Time interval is %2d seconds:\n",BERInterval);               
            
       break; /* Case 9 */

	 case 'l':
     case 'L': /* Loop reset */
       NOK = 0;
       for (j = 0; j < 5000; j++)
       {
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {
        Control_Reg[i] = Control_Reg[i] | (0x8000);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        usleep(10);
        
        
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {      
        Control_Reg[i] = Control_Reg[i] & (0x7FFF);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        do
        {
          Counter_1ms_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
        } while (Counter_1ms_Reg[0] < 10);
        
        Channel_Reg[0]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL_REG_BASE);
        ChannelOK[0]  = 0x0001 & (Channel_Reg[0] >> 3);     
		
		if (ChannelOK[0] == 0) 
			{
			NOK = NOK + 1;
			printf("\nLink NOK  press any key: %d :\n",NOK);
			rx_char = input_char(); 

			}
		else
			printf(" %d",j);
			
        } 
        printf("\n\nTotal NOK : %d", NOK);
        break; //case 'l'         
 

      case 'a': 
		case 'A':/*Select Channel To Control*/

      printf("A. Select Channel To Control for EyeQ\n");
      printf("Current Channel : ");
      switch (SelectedChannel)
       {
       case  0  : printf(" 0: GXB_RXTX[12] SMA   \n"); break;
       case  1  : printf(" 1: GXB_RXTX[13] SMA   \n"); break;
       case  2  : printf(" 2: GXB_RXTX[14] SMA   \n"); break;
       case  3  : printf(" 3: GXB_RXTX[15] SMA   \n"); break;		 
       default : break;
       }  
     
     temp = 0xffffffff;
  
     do {
      printf("New Channel (0-4)         :");
      rx_char = input_char(); 
  
      switch (rx_char)
      {
      case '0': temp = 0; break;
      case '1': temp = 1; break;
      case '2': temp = 2; break;
      case '3': temp = 3; break;		
      default : break;
      }
  
    } while (temp == 0xffffffff);
    
    SelectedChannel = temp;
         
                   

        
      break; //case 'a'

      case 'g':
      case 'G': /* New EyeQ Interval Time Interval */
        
        printf("G. Provide EyeQ Time Interval\n");
        printf("   Insert a Hexadecimal Number from 01 to FF\n");
        printf("   Current BER Time Interval   : %4d miliseconds\n", EyeQInterval);
  
		
        temp = 0xffffffff;
           do {
            printf("\n   New Interval (Higher Nibble) (0-F)        :");
            rx_char   = input_char();
        
            switch (rx_char)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          } while (temp == 0xffffffff);
          
          High_nibble = temp;
          
          temp = 0xffffffff;
           do {
            printf("\n   New Interval (Lower Nibble) (0-F)        :");
            rx_char   = input_char();
        
            switch (rx_char)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          } while (temp == 0xffffffff);
          
          Low_nibble = temp;  
    
          EyeQInterval = (High_nibble << 4) | Low_nibble;
         
            
          printf("\n    New EyeQTime interval is %4d miliseconds:\n",EyeQInterval);               
            
       break; /* Case G */
	 
	 
		case 'm':
		case 'M':
		printf("\n");
		
		//eye1d = 0;	
		//eyeqbw = 3; //Datarate > 7.5  Gbps 
		
		write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0007); // Enable Eye Monitor	and BERB Block	
		
		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
		if (DEBUG==1)
				printf("\nEyeQstatus = %4x\n",eyeq_status);
		eyeq_control = eyeq_status;
		//eyeq_control = (eyeq_control & (0x0000DFFF)) | (eye1d << 13);
		//eyeq_control = (eyeq_control & (0x0000F9FF)) | (eyeqbw << 9);
		
		//write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); // Set eye1d to 0 and Eyeq Bandwidth
		
		eyepolarity = TOP;
		eyeq_control = (eyeq_control & (0x0000FFFB)) | (eyepolarity << 2);	
		write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); // Set eyepolarity
		
		if (DEBUG==1)
			printf("\neyeq_control = %4x\n",eyeq_control);		
			

		
	    ///////////////////////////////////////////////////////////////////////				
		// Go through all horizontal phase steps and all vertical phase steps one by one and measure the BER
		///////////////////////////////////////////////////////////////////////	
		
		// Upper half first
            
		for (x = 0; x < 32 ; x++)
			{
				for (y = 0; y < 64 ; y++)
					{ 
			
			write_reconfiguration_EyeQ(SelectedChannel,0x2,y); // Select vertical phase
			usleep(10); // Wait for 10 us to stabilize the vertical level to settle down
						
			write_reconfiguration_EyeQ(SelectedChannel,0x1,x); // Select horizontal phase
							
			usleep(100); // wait for Prbslogic to stabilize
			
		   
		   write_reconfiguration_EyeQ(SelectedChannel,0x0,0x000F); // Reset counters
					   

			// Reset Counter_1MS register

	     Control_Reg[0] = Control_Reg[0] | (0x2000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        Control_Reg[0] = Control_Reg[0] & (0xDFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);	
						
			TimeInterval = EyeQInterval; // EyeQInterval is in ms.
	
			// Wait for programmed time interval
			do
			{
			Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
			} while (Counter_1ms_Reg[0] < TimeInterval);                  
             
			
         write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0017); // Take a snapshot
			
			
  
				ErrorCount_Reg_L[SelectedChannel]     = read_reconfiguration_EyeQ(SelectedChannel,0x7);
				ErrorCount_Reg_H[SelectedChannel]     = read_reconfiguration_EyeQ(SelectedChannel,0x8);
	
				BitCount_L[SelectedChannel]     		  = read_reconfiguration_EyeQ(SelectedChannel,0x5);
				BitCount_H[SelectedChannel]     		  = read_reconfiguration_EyeQ(SelectedChannel,0x6);			
				
			
				ErrorCount_Array[x][y] = ((float) (ErrorCount_Reg_H[SelectedChannel]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[SelectedChannel]);
	
			   Totalbits[0] = ((float) (BitCount_H[SelectedChannel] ) *  MULTIPLIER * MULTIPLIER) + (float) (BitCount_L[SelectedChannel] );
				
				//Totalbits[0] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[0]*1000);
	  
				if (ErrorCount_Array[x][y] > 0)
					{
				  BER_Array[x][y] = ((ErrorCount_Array[x][y])) / Totalbits[0];
				  //printf("\nErrorCount %6d %6d",ErrorCount_Reg_H[SelectedChannel],ErrorCount_Reg_L[SelectedChannel]);
				  //printf("ErrorCount : %5e\n", ErrorCount_Array[i]);
				  }
				else 
				  BER_Array[x][y] = 0;  
				  
			
																																				
		} // for loop y
		
	} // for loop x

		// Bottom half
      
		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
		if (DEBUG==1)	
			printf("\nEyeQstatus = %4x\n",eyeq_status);
		eyeq_control = eyeq_status;
	
		eyepolarity = BOTTOM;
		eyeq_control = (eyeq_control & (0x0000FFFB)) | (eyepolarity << 2);	
		write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); 
		if (DEBUG==1)
			printf("\neyeq_control = %4x\n",eyeq_control);	
	
		//write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0001); // Enable Eye Monitor
	
		for (x = 0; x < 32 ; x++)
			{
				for (y = 1; y < 64 ; y++)
					{ 
			write_reconfiguration_EyeQ(SelectedChannel,0x2,y); // Select vertical phase
			usleep(10); // Wait for 10 us to stabilize the vertical level to settle down
						
			write_reconfiguration_EyeQ(SelectedChannel,0x1,x); // Select horizontal phase
							
			usleep(100); // wait for Logic to stabilize
			
		   
		   write_reconfiguration_EyeQ(SelectedChannel,0x0,0x000F); // Reset counters

			// Reset Counter_1MS register

	     Control_Reg[0] = Control_Reg[0] | (0x2000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        Control_Reg[0] = Control_Reg[0] & (0xDFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
					   
		
			TimeInterval = EyeQInterval; // EyeQInterval is in ms.
	
			// Wait for programmed time interval
			do
			{
			Counter_1ms_Reg[0]     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
			} while (Counter_1ms_Reg[0] < TimeInterval);                  
             
			
         write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0017); // Take a snapshot
			
			
  
				ErrorCount_Reg_L[SelectedChannel]     = read_reconfiguration_EyeQ(SelectedChannel,0x7);
				ErrorCount_Reg_H[SelectedChannel]     = read_reconfiguration_EyeQ(SelectedChannel,0x8);
	
				BitCount_L[SelectedChannel]     		  = read_reconfiguration_EyeQ(SelectedChannel,0x5);
				BitCount_H[SelectedChannel]     		  = read_reconfiguration_EyeQ(SelectedChannel,0x6);			
				
			
				ErrorCount_Array_Bottom[x][y] = ((float) (ErrorCount_Reg_H[SelectedChannel]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[SelectedChannel]);
	
			   Totalbits[0] = ((float) (BitCount_H[SelectedChannel] ) *  MULTIPLIER * MULTIPLIER) + (float) (BitCount_L[SelectedChannel] );
				
				//Totalbits[0] = (float) (Counter_1ms_Reg[0]) * (float) (Bitrate[0]*1000);

	  
				if (ErrorCount_Array_Bottom[x][y] > 0)
					{
				  BER_Array_Bottom[x][y] = ((ErrorCount_Array_Bottom[x][y])) / Totalbits[0];
				  //printf("\nErrorCount %6d %6d",ErrorCount_Reg_H[SelectedChannel],ErrorCount_Reg_L[SelectedChannel]);
				  //printf("ErrorCount : %5e\n", ErrorCount_Array_Bottom[x][y]);
				  }
				else 
				  BER_Array_Bottom[x][y] = 0;  
				  
//				if (ErrorCount_Array_Bottom[x][y] > 0)
//						printf("Ch %1d Phase Step : %2d %2d BER Estimate : %4e    ErrorCount  : %4e \n",SelectedChannel,x,y,(BER_Array_Bottom[x][y]),ErrorCount_Array_Bottom[x][y]);
//					else
//						printf("Ch %1d Phase Step : %2d %2d BER          : 0\n",SelectedChannel,x,y);			    
            
			
																																				
		} // for loop y
		
	} // for loop x
	
		//printf("\n BATHTUB CURVE ESTIMATE ACROSS 64 PHASES\n");
	    printf("\n 2-D EyeMargin ESTIMATE ACROSS 32 PHASES (one UI) using BERB\n");
       
	       ///////////////////////////////////////////////////////////////////////				
		   // Determine eye opening
           ///////////////////////////////////////////////////////////////////////				
	
		eye_phase = 0;
		BER_Minimum = 1;
		BER_Minimum_phase = 0;
		 
        for (i = 0; i <32 ;i++)
        {
			if (BER_Array[i][0] == 0)
               eye_phase = eye_phase + 1;
            if (BER_Array[i][0] < BER_Minimum )
				{
				BER_Minimum = BER_Array[i][0];
				BER_Minimum_phase = i;
				}
        }
        
			

         eye = (float) eye_phase/32;
        
        start_value = 0;
        end_value = 0;
        
        for (i = 1; i <32 ;i++)
        {
          if (BER_Array[i][0] != BER_Array[i-1][0])
          {
               if (BER_Array[i][0] == 0)
					start_value = i;
               else if (BER_Array[i-1][0] == 0)
                     end_value = i-1;
           }
        }
        //printf("\nstart_value : %2d",start_value);
        //printf("\nend_value   : %2d",end_value);
        if (start_value > end_value )
			end_value = end_value + 32;
		

		//printf("\nend_value   : %2d",end_value);	
		if (eye_phase == 0)
			optimum_phase = BER_Minimum_phase ;
		else	
			optimum_phase = start_value + eye_phase/2;
			
		//printf("\noptimum phase   : %2d",optimum_phase );
		if (optimum_phase >= 32 )
			{
			table_phase_start = optimum_phase - 15;
			optimum_phase = optimum_phase -32;
			}
		else
			{
			table_phase_start = optimum_phase -15;
			if (table_phase_start < 0)
				table_phase_start = table_phase_start + 32;
			}
		//printf("\noptimum phase   : %2d",optimum_phase );
		//printf("\ntable_phase_start   : %2d",table_phase_start );	

	   ///////////////////////////////////////////////////////////////////////				
	   // Find Vertical Eye Opening using optimum horizontal phase
	   ///////////////////////////////////////////////////////////////////////					
				
		veye = 0;
		 
        for (i = 0; i <64 ;i++)
        {
			if (BER_Array[optimum_phase][i] == 0)
               veye = veye + 1;
        }	
        for (i = 0; i <64 ;i++)
        {
			if (BER_Array_Bottom[optimum_phase][i] == 0)
               veye = veye + 1;
        }		  
		
		veye_top = veye/2 + 10;
		veye_bottom = veye/2 + 10;  
		
			
	   ///////////////////////////////////////////////////////////////////////				
	   // Print Centered Eye
	   ///////////////////////////////////////////////////////////////////////				
       printf("\nPhase   ");
       for (i =0;i <32; i++)
			{
			index = table_phase_start + i;
			if (index > 31)
				index = index -32;
			if (index < 10) 
				printf("  ");
			else if ((index >= 10) && (index < 20))
			    printf(" 1");
			else if ((index >= 20) && (index < 30))
				printf(" 2");
			else
				printf(" 3");
			}
	  printf("\nStep    ");
       for (i =0;i <32; i++)
			{
			index = table_phase_start + i;
			if (index > 31)
				index = index -32;
			if (index < 10) 
				printf("%2d",index);
			else if ((index >= 10) && (index < 20))
			    printf("%2d",index-10);
			else if ((index >= 20) && (index < 30))
				printf("%2d",index-20);
			else
				printf("%2d",index-30);
			}			
	   printf("\n========================================================================\n");	   	   
	   for (j = veye_top; j >= 0 ; j--)
			{   
			printf("\n%2d     :",j);  
				for (i = 0; i < 32 ; i++)
				{ 
					index = table_phase_start + i;
					if (index > 31)
						index = index -32;				    
					if (BER_Array[index][j] == 0)
						printf(" .");
					else
						printf(" x");
				}
			}
	   for (j = 1; j < veye_bottom ; j++)
			{   
			printf("\n-%2d    :",j);  
				for (i = 0; i < 32 ; i++)
				{ 
					index = table_phase_start + i;
					if (index > 31)
						index = index -32;				    
					if (BER_Array_Bottom[index][j] == 0)
						printf(" .");
					else
						printf(" x");
				}
			}		
		printf("\n");				
                
         printf("\nHorizontal Eye Opening Ch %1d: %2d phases",SelectedChannel,eye_phase);
         printf("\nHorizontal Eye Opening Ch %1d: %.2f UI",SelectedChannel,eye);
         printf("\nOptimum Phase Ch %1d         : %2d",SelectedChannel,optimum_phase);
         printf("\nVertical Eye Opening Ch %1d  : %2d steps",SelectedChannel,veye);	
			printf("\nSampling time per sample     : %2d ms",EyeQInterval);	
			
			
	// Set Eyepolarity to TOP again
	
		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
		if (DEBUG==1)			
			printf("\nEyeQstatus = %4x\n",eyeq_status);
		eyeq_control = eyeq_status;
			
	eyepolarity = TOP;
	eyeq_control = (eyeq_control & (0x0000FFFB)) | (eyepolarity << 2);	
	write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control);
			
   write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0000); // Disable Eye Monitor			
		
		
		break;

	 case 't':
     case 'T': /* Loop to measure latency and delay */
       NOK = 0;
	   Latency_Minimum = 1000;
		Latency_Maximum = 0;

       for (j = 0; j < 8; j++)
       {
				Latency_memory[j] = 0;
		 }
	  
       for (j = 0; j < RESET_CYCLES; j++)
       {
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {
        Control_Reg[i] = Control_Reg[i] | (0x8000);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        usleep(10);
        
        
        for (i = 0; i < NUMBER_OF_QUADS ; i++)
        {      
        Control_Reg[i] = Control_Reg[i] & (0x7FFF);
        }
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg[0]);
        
        do
        {
          Counter_1ms_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
        } while (Counter_1ms_Reg[0] < 10);

        do
        {
			Channel_Reg[0]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL_REG_BASE);
			ChannelOK[0]  = 0x0001 & (Channel_Reg[0] >> 3);    
        } while (ChannelOK[0] = 0);
		  
		  // Measure time how long it takes to reach ChannelOK
        Counter_1ms_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
		  
        Channel_Reg[0]      = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL_REG_BASE);
        ChannelOK[0]  = 0x0001 & (Channel_Reg[0] >> 3);    

		Latency_Reg[0] = IORD_ALTERA_AVALON_PIO_DATA(LATENCY_REG_BASE);
		Latency_measure_temp[0] = ((float)Latency_Reg[0] * 1000000000)/((float)DataClock_Reg[0]);
		Latency_measure[0] = (Latency_measure_temp[0]); // Convert the floats to integer numbers  
		
		  
		  
		if (ChannelOK[0] == 0) 
			{
			NOK = NOK + 1;
 
			if (Latency_measure[0] < Latency_Minimum )
				Latency_Minimum = Latency_measure[0];
			if (Latency_measure[0] > Latency_Maximum )
				Latency_Maximum = Latency_measure[0];

						switch (Latency_Reg[0])
							{
							case 20 : Latency_memory[0] = Latency_memory[0] + 1;break;									
							case 21 : Latency_memory[1] = Latency_memory[1] + 1;break;						
							case 22 : Latency_memory[2] = Latency_memory[2] + 1;break;	
							case 23 : Latency_memory[3] = Latency_memory[3] + 1;break;	
							case 24 : Latency_memory[4] = Latency_memory[4] + 1;break;		
							case 25 : Latency_memory[5] = Latency_memory[5] + 1;break;	
							case 26 : Latency_memory[6] = Latency_memory[6] + 1;break;	
							case 27 : Latency_memory[7] = Latency_memory[7] + 1;break;									
							default : break;	
							}		
		  
						printf("\n");
					   printf("%4d:Latency:%3d ns",j,Latency_measure[0]);	
			if (STOP == 1)
			{
				printf("\nLink NOK  press any key: %d :\n",NOK);
				rx_char = input_char();
			}
			}
		else
		{
			if (Latency_measure[0] < Latency_Minimum )
				Latency_Minimum = Latency_measure[0];
			if (Latency_measure[0] > Latency_Maximum )
				Latency_Maximum = Latency_measure[0];

						switch (Latency_Reg[0])
							{
							case 20 : Latency_memory[0] = Latency_memory[0] + 1;break;									
							case 21 : Latency_memory[1] = Latency_memory[1] + 1;break;						
							case 22 : Latency_memory[2] = Latency_memory[2] + 1;break;	
							case 23 : Latency_memory[3] = Latency_memory[3] + 1;break;	
							case 24 : Latency_memory[4] = Latency_memory[4] + 1;break;		
							case 25 : Latency_memory[5] = Latency_memory[5] + 1;break;	
							case 26 : Latency_memory[6] = Latency_memory[6] + 1;break;	
							case 27 : Latency_memory[7] = Latency_memory[7] + 1;break;									
							default : break;	
							}			
		  
						printf("\n");
						printf("%4d:Latency:%3d ns ",j,Latency_measure[0]);	
        }//  if-else
		  
	  } // for loop
	
	       for (j = 0; j < HISTOGRAM_SIZE; j++)
       {  
	  		Latency_memory_measure_temp[j] = ((float)(20+j) * 1000000000)/((float)DataClock_Reg[0]);
		   Latency_memory_measure[j] = (Latency_memory_measure_temp[j]); // Convert the floats to integer numbers  
		 }
		 
        printf("\n\nTotal NOK : %d", NOK);
	     printf("\nTotal OK : %d", (RESET_CYCLES-NOK));
	     printf("\nMinimum Latency over %d reset cycles : %d ns",(RESET_CYCLES-NOK),Latency_Minimum);
	     printf("\nMaximum Latency over %d reset cycles : %d ns",(RESET_CYCLES-NOK),Latency_Maximum);  
	       for (j = 0; j < HISTOGRAM_SIZE; j++)
       {  		 
	     printf("\nNumber of cases Latency Equals %d ns : %d",Latency_memory_measure[j],Latency_memory[j]); 
	    }
        break; //case 'T'         
		
              
     case '0':
        printf("\nTerminating Program.\n");
        return 0;
       break;
  
      default:
        printf("\n\tPlease enter an integer value from 0 to A\n\n\n");
        break;
    } //Case rx_char
 } // While
 
 return 0;
} // Main
